@@ -4,9 +4,8 @@ Parser for HTML forms, that fills in defaults and errors.  See
 """
 
 import HTMLParser
-import cgi
 import re
-from htmlentitydefs import name2codepoint
+from formencode.rewritingparser import RewritingParser, html_quote
 
 __all__ = ['render', 'htmlliteral', 'default_formatter',
            'none_formatter', 'escape_formatter',
@@ -16,7 +15,8 @@ def render(form, defaults=None, errors=None, use_all_keys=False,
            error_formatters=None, add_attributes=None,
            auto_insert_errors=True, auto_error_formatter=None,
            text_as_default=False, listener=None, encoding=None,
-           error_class='error'):
+           error_class='error', prefix_error=True,
+           force_defaults=True):
     """
     Render the ``form`` (which should be a string) given the defaults
     and errors.  Defaults are the values that go in the input fields
@@ -57,6 +57,16 @@ def render(form, defaults=None, errors=None, use_all_keys=False,
     
     ``encoding`` specifies an encoding to assume when mixing str and 
     unicode text in the template.
+    
+    ``prefix_error`` specifies if the HTML created by auto_error_formatter is
+    put before the input control (default) or after the control.
+    
+    ``force_defaults`` specifies if a field default is not given in
+    the ``defaults`` dictionary then the control associated with the
+    field should be set as an unsuccessful control. So checkboxes will
+    be cleared, radio and select controls will have no value selected,
+    and textareas will be emptied. This defaults to ``True``, which is
+    appropriate the defaults are the result of a form submission.
     """
     if defaults is None:
         defaults = {}
@@ -70,7 +80,9 @@ def render(form, defaults=None, errors=None, use_all_keys=False,
         auto_error_formatter=auto_error_formatter,
         text_as_default=text_as_default,
         listener=listener, encoding=encoding,
+        prefix_error=prefix_error,
         error_class=error_class,
+        force_defaults=force_defaults,
         )
     p.feed(form)
     p.close()
@@ -93,22 +105,12 @@ class htmlliteral(object):
         return self.text
 
     def __repr__(self):
-        return '<%s html=%r text=%r>' % (self.html, self.text)
+        return '<%s html=%r text=%r>' % (
+            self.__class__.__name__, self.html, self.text)
 
     def __html__(self):
         return self.html
 
-def html_quote(v):
-    if v is None:
-        return ''
-    elif hasattr(v, '__html__'):
-        return v.__html__()
-    elif isinstance(v, basestring):
-        return cgi.escape(v, 1)
-    else:
-        # @@: Should this be unicode(v) or str(v)?
-        return cgi.escape(str(v), 1)
-
 def default_formatter(error):
     """
     Formatter that escapes the error, wraps the error in a span with
@@ -136,7 +138,13 @@ def escapenl_formatter(error):
     error = error.replace('\n', '<br>\n')
     return error
 
-class FillingParser(HTMLParser.HTMLParser):
+def ignore_formatter(error):
+    """
+    Formatter that emits nothing, regardless of the error.
+    """
+    return ''
+
+class FillingParser(RewritingParser):
     r"""
     Fills HTML with default values, as in a form.
 
@@ -151,35 +159,41 @@ class FillingParser(HTMLParser.HTMLParser):
         >>> parser.feed('<input type="text" name="name" value="fill">\
         ... <select name="occupation"><option value="">Default</option>\
         ... <option value="Crazy Cultist">Crazy cultist</option>\
-        ... </select> <textarea cols=20 style="width: 100%" name="address">An address\
+        ... </select> <textarea cols="20" style="width: 100%" name="address">An address\
         ... </textarea> <input type="radio" name="living" value="yes">\
         ... <input type="radio" name="living" value="no">\
         ... <input type="checkbox" name="nice_guy" checked="checked">')
-        >>> print parser.text()
+        >>> parser.close()
+        >>> print parser.text() # doctest: +NORMALIZE_WHITESPACE
         <input type="text" name="name" value="Bob Jones">
         <select name="occupation">
         <option value="">Default</option>
         <option value="Crazy Cultist" selected="selected">Crazy cultist</option>
         </select>
-        <textarea cols=20 style="width: 100%" name="address">14 W. Canal
+        <textarea cols="20" style="width: 100%" name="address">14 W. Canal
         New Guinea</textarea>
         <input type="radio" name="living" value="yes">
-        <input type="radio" name="living" value="no" selected="selected">
+        <input type="radio" name="living" value="no" checked="checked">
         <input type="checkbox" name="nice_guy">
+
     """
 
+    default_encoding = 'utf8'
+
     def __init__(self, defaults, errors=None, use_all_keys=False,
                  error_formatters=None, error_class='error',
                  add_attributes=None, listener=None,
                  auto_error_formatter=None,
-                 text_as_default=False, encoding=None):
-        HTMLParser.HTMLParser.__init__(self)
-        self._content = []
+                 text_as_default=False, encoding=None, prefix_error=True,
+                 force_defaults=True):
+        RewritingParser.__init__(self)
         self.source = None
         self.lines = None
         self.source_pos = None
         self.defaults = defaults
         self.in_textarea = None
+        self.skip_textarea = False
+        self.last_textarea_name = None
         self.in_select = None
         self.skip_next = False        
         self.errors = errors or {}
@@ -200,19 +214,30 @@ class FillingParser(HTMLParser.HTMLParser):
         self.auto_error_formatter = auto_error_formatter
         self.text_as_default = text_as_default
         self.encoding = encoding
-
-    def feed(self, data):
-        self.data_is_str = isinstance(data, str)
-        self.source = data
-        self.lines = data.split('\n')
-        self.source_pos = 1, 0
-        if self.listener:
-            self.listener.reset()
-        HTMLParser.HTMLParser.feed(self, data)
+        self.prefix_error = prefix_error
+        self.force_defaults = force_defaults
+    
+    def str_compare(self, str1, str2):
+        """
+        Compare the two objects as strings (coercing to strings if necessary).
+        Also uses encoding to compare the strings.
+        """
+        if not isinstance(str1, basestring):
+            if hasattr(str1, '__unicode__'):
+                str1 = unicode(str1)
+            else:
+                str1 = str(str1)
+        if type(str1) == type(str2):
+            return str1 == str2
+        if isinstance(str1, unicode):
+            str1 = str1.encode(self.encoding or self.default_encoding)
+        else:
+            str2 = str2.encode(self.encoding or self.default_encoding)
+        return str1 == str2
 
     def close(self):
         self.handle_misc(None)
-        HTMLParser.HTMLParser.close(self)
+        RewritingParser.close(self)
         unused_errors = self.errors.copy()
         for key in self.used_errors.keys():
             if unused_errors.has_key(key):
@@ -246,50 +271,14 @@ class FillingParser(HTMLParser.HTMLParser):
                     item = item.decode(self.encoding)
                 new_content.append(item)
             self._content = new_content
-        try:
-            self._text = ''.join([
-                t for t in self._content if not isinstance(t, tuple)])
-        except UnicodeDecodeError, e:
-            if self.data_is_str:
-                e.reason += (
-                    " the form was passed in as an encoded string, but "
-                    "some data or error messages were unicode strings; "
-                    "the form should be passed in as a unicode string")
-            else:
-                e.reason += (
-                    " the form was passed in as an unicode string, but "
-                    "some data or error message was an encoded string; "
-                    "the data and error messages should be passed in as "
-                    "unicode strings")
-            raise
+        self._text = self._get_text()
+
+    def skip_output(self):
+        return (self.in_textarea and self.skip_textarea) or self.skip_error
 
     def add_key(self, key):
         self.used_keys[key] = 1
 
-    _entityref_re = re.compile('&([a-zA-Z][-.a-zA-Z\d]*);')
-    _charref_re = re.compile('&#(\d+|[xX][a-fA-F\d]+);')
-
-    def unescape(self, s):
-        s = self._entityref_re.sub(self._sub_entityref, s)
-        s = self._charref_re.sub(self._sub_charref, s)
-        return s
-
-    def _sub_entityref(self, match):
-        name = match.group(1)
-        if name not in name2codepoint:
-            # If we don't recognize it, pass it through as though it
-            # wasn't an entity ref at all
-            return match.group(0)
-        return unichr(name2codepoint[name])
-
-    def _sub_charref(self, match):
-        num = match.group(1)
-        if num.lower().startswith('x'):
-            num = int(num[1:], 16)
-        else:
-            num = int(num)
-        return unichr(num)
-
     def handle_starttag(self, tag, attrs, startend=False):
         self.write_pos()
         if tag == 'input':
@@ -312,16 +301,6 @@ class FillingParser(HTMLParser.HTMLParser):
         if self.listener:
             self.listener.listen_input(self, tag, attrs)
 
-    def handle_misc(self, whatever):
-        self.write_pos()
-    handle_charref = handle_misc
-    handle_entityref = handle_misc
-    handle_data = handle_misc
-    handle_comment = handle_misc
-    handle_decl = handle_misc
-    handle_pi = handle_misc
-    unknown_decl = handle_misc
-
     def handle_endtag(self, tag):
         self.write_pos()
         if tag == 'textarea':
@@ -372,7 +351,8 @@ class FillingParser(HTMLParser.HTMLParser):
     def handle_input(self, attrs, startend):
         t = (self.get_attr(attrs, 'type') or 'text').lower()
         name = self.get_attr(attrs, 'name')
-        self.write_marker(name)
+        if self.prefix_error:
+            self.write_marker(name)
         value = self.defaults.get(name)
         if self.add_attributes.has_key(name):
             for attr_name, attr_value in self.add_attributes[name].items():
@@ -387,17 +367,21 @@ class FillingParser(HTMLParser.HTMLParser):
             and self.errors.get(self.get_attr(attrs, 'name'))):
             self.add_class(attrs, self.error_class)
         if t in ('text', 'hidden'):
-            if value is None:
+            if value is None and not self.force_defaults:
                 value = self.get_attr(attrs, 'value', '')
             self.set_attr(attrs, 'value', value)
             self.write_tag('input', attrs, startend)
             self.skip_next = True
             self.add_key(name)
         elif t == 'checkbox':
-            selected = False
+            if self.force_defaults:
+                selected = False
+            else:
+                selected = self.get_attr(attrs, 'checked')
             if not self.get_attr(attrs, 'value'):
                 selected = value
-            elif self.selected_multiple(value, self.get_attr(attrs, 'value')):
+            elif self.selected_multiple(value,
+                                        self.get_attr(attrs, 'value', '')):
                 selected = True
             if selected:
                 self.set_attr(attrs, 'checked', 'checked')
@@ -407,9 +391,9 @@ class FillingParser(HTMLParser.HTMLParser):
             self.skip_next = True
             self.add_key(name)
         elif t == 'radio':
-            if str(value) == self.get_attr(attrs, 'value'):
+            if self.str_compare(value, self.get_attr(attrs, 'value', '')):
                 self.set_attr(attrs, 'checked', 'checked')
-            else:
+            elif self.force_defaults or name in self.defaults:
                 self.del_attr(attrs, 'checked')
             self.write_tag('input', attrs, startend)
             self.skip_next = True
@@ -417,14 +401,13 @@ class FillingParser(HTMLParser.HTMLParser):
         elif t == 'file':
             pass # don't skip next
         elif t == 'password':
-            self.set_attr(attrs, 'value', value or
-                          self.get_attr(attrs, 'value', ''))
+            if value is None and not self.force_defaults:
+                value = value or self.get_attr(attrs, 'value', '')
+            self.set_attr(attrs, 'value', value)
             self.write_tag('input', attrs, startend)
             self.skip_next = True
             self.add_key(name)
         elif t == 'image':
-            self.set_attr(attrs, 'src', value or
-                          self.get_attr(attrs, 'src', ''))
             self.write_tag('input', attrs, startend)
             self.skip_next = True
             self.add_key(name)
@@ -444,27 +427,40 @@ class FillingParser(HTMLParser.HTMLParser):
         else:
             assert 0, "I don't know about this kind of <input>: %s (pos: %s)" \
                    % (t, self.getpos())
+        if not self.prefix_error:
+            self.write_marker(name)
 
     def handle_textarea(self, attrs):
         name = self.get_attr(attrs, 'name')
-        self.write_marker(name)
+        if self.prefix_error:
+            self.write_marker(name)
         if (self.error_class
             and self.errors.get(name)):
             self.add_class(attrs, self.error_class)
-        self.write_tag('textarea', attrs)
         value = self.defaults.get(name, '')
-        self.write_text(html_quote(value))
-        self.write_text('</textarea>')
+        if value or self.force_defaults:
+            self.write_tag('textarea', attrs)
+            self.write_text(html_quote(value))
+            self.write_text('</textarea>')
+            self.skip_textarea = True
         self.in_textarea = True
+        self.last_textarea_name = name
         self.add_key(name)
 
     def handle_end_textarea(self):
+        if self.skip_textarea:
+            self.skip_textarea = False
+        else:
+            self.write_text('</textarea>')            
         self.in_textarea = False
         self.skip_next = True
+        if not self.prefix_error:
+            self.write_marker(self.last_textarea_name)
+        self.last_textarea_name = None
 
     def handle_select(self, attrs):
         name = self.get_attr(attrs, 'name', False)
-        if name:
+        if name and self.prefix_error:
             self.write_marker(name)
         if (self.error_class
             and self.errors.get(name)):
@@ -475,6 +471,10 @@ class FillingParser(HTMLParser.HTMLParser):
         self.add_key(self.in_select)
 
     def handle_end_select(self):
+        self.write_text('</select>')
+        self.skip_next = True
+        if not self.prefix_error and self.in_select:
+            self.write_marker(self.in_select)
         self.in_select = None
 
     def handle_option(self, attrs):
@@ -483,13 +483,13 @@ class FillingParser(HTMLParser.HTMLParser):
             % self.getpos())
         if self.in_select != False:
             default = self.defaults.get(self.in_select, '')
-
-            if self.selected_multiple(self.defaults.get(self.in_select, ''),
-                                      self.get_attr(attrs, 'value')):
-                self.set_attr(attrs, 'selected', 'selected')
-                self.add_key(self.in_select)
-            else:
-                self.del_attr(attrs, 'selected')
+            if self.force_defaults:
+                if self.selected_multiple(self.defaults.get(self.in_select, ''),
+                                          self.get_attr(attrs, 'value', '')):
+                    self.set_attr(attrs, 'selected', 'selected')
+                    self.add_key(self.in_select)
+                else:
+                    self.del_attr(attrs, 'selected')
         self.write_tag('option', attrs)
         self.skip_next = True
 
@@ -508,12 +508,9 @@ class FillingParser(HTMLParser.HTMLParser):
                 return True
         if hasattr(obj, '__iter__'):
             for inner in obj:
-                if str(inner) == value:
+                if self.str_compare(inner, value):
                     return True
-        return str(obj) == value
-
-    def write_text(self, text):
-        self._content.append(text)
+        return self.str_compare(obj, value)
 
     def write_marker(self, marker):
         self._content.append((marker,))
@@ -526,70 +523,10 @@ class FillingParser(HTMLParser.HTMLParser):
         else:
             self._content.insert(0, text)
 
-    def write_tag(self, tag, attrs, startend=False):
-        attr_text = ''.join([' %s="%s"' % (n, html_quote(v))
-                             for (n, v) in attrs
-                             if not n.startswith('form:')])
-        if startend:
-            attr_text += " /"
-        self.write_text('<%s%s>' % (tag, attr_text))
-
-    def write_pos(self):
-        cur_line, cur_offset = self.getpos()
-        if self.in_textarea or self.skip_error:
-            self.source_pos = self.getpos()
-            return
-        if self.skip_next:
-            self.skip_next = False
-            self.source_pos = self.getpos()
-            return
-        if cur_line == self.source_pos[0]:
-            self.write_text(
-                self.lines[cur_line-1][self.source_pos[1]:cur_offset])
-        else:
-            self.write_text(
-                self.lines[self.source_pos[0]-1][self.source_pos[1]:])
-            self.write_text('\n')
-            for i in range(self.source_pos[0]+1, cur_line):
-                self.write_text(self.lines[i-1])
-                self.write_text('\n')
-            self.write_text(self.lines[cur_line-1][:cur_offset])
-        self.source_pos = self.getpos()
-
-    def get_attr(self, attr, name, default=None):
-        for n, value in attr:
-            if n.lower() == name:
-                return value
-        return default
-
-    def set_attr(self, attr, name, value):
-        for i in range(len(attr)):
-            if attr[i][0].lower() == name:
-                attr[i] = (name, value)
-                return
-        attr.append((name, value))
-
-    def del_attr(self, attr, name):
-        for i in range(len(attr)):
-            if attr[i][0].lower() == name:
-                del attr[i]
-                break
-
-    def add_class(self, attr, class_name):
-        current = self.get_attr(attr, 'class', '')
-        new = current + ' ' + class_name
-        self.set_attr(attr, 'class', new.strip())
-            
-    def text(self):
-        try:
-            return self._text
-        except AttributeError:
-            raise Exception(
-                "You must .close() a parser instance before getting "
-                "the text from it")
-
 # This can potentially be extended globally
 default_formatter_dict = {'default': default_formatter,
                           'none': none_formatter,
                           'escape': escape_formatter,
-                          'escapenl': escapenl_formatter}
+                          'escapenl': escapenl_formatter,
+                          'ignore': ignore_formatter,
+                          }
