@@ -9,11 +9,11 @@ def validate(month, year):
     except Invalid, e:
         return e.unpack_errors()['ccExpiresMonth']
 
-messages = CreditCardExpires._messages
+messages = ed.message
 
 def test_ed():
     assert validate('11', '2250') is None
-    assert validate('11', 'test') == messages['notANumber']
-    assert validate('test', '2250') == messages['notANumber']
-    assert validate('10', '2005') == messages['invalidNumber']
-    assert validate('10', '05') == messages['invalidNumber']
+    assert validate('11', 'test') == messages('notANumber', None)
+    assert validate('test', '2250') == messages('notANumber', None)
+    assert validate('10', '2005') == messages('invalidNumber', None)
+    assert validate('10', '05') == messages('invalidNumber', None)
