@@ -66,8 +66,8 @@ def test_es():
 def test_pt_BR():
   _test_lang("pt_BR", u"Por favor digite um valor")
 
-def test_big5():
-  _test_lang("big5", u"請輸入一個值")
+def test_zh_TW():
+  _test_lang("zh_TW", u"請輸入一個值")
 
 def test_sk():
   _test_lang("sk",u"Zadajte hodnotu, prosím")
@@ -82,7 +82,7 @@ def test_pt_PT():
   _test_lang("pt_PT", u"Por favor insira um valor")
 
 def test_fr():
-  _test_lang("fr", u"Veuillez entrer une valeur")
+  _test_lang("fr", u"Saisissez une valeur")
 
 def test_nl():
   _test_lang("nl", u"Voer een waarde in")
@@ -108,3 +108,14 @@ def test_nb_NO():
 def test_it():
   _test_lang("it", u"Inserire un dato")
 
+def test_et():
+  _test_lang("et", u"Palun sisestada väärtus")
+
+def test_lt():
+  _test_lang("lt", u"Prašome įvesti reikšmę")
+
+def test_ja():
+  _test_lang("ja", u"入力してください")
+
+def test_tr():
+  _test_lang("tr", u"Lütfen bir değer giriniz")
